import os
import logging
import time
import md5
import cPickle
import WebFrame as WF

_sidhelper = None  # See notes at bottom

class Session(dict):
    def __init__(self, ctx):
        dict.__init__(self)
        
        global _sidhelper
        try:
            self.sid = ctx.req.get_cookie("WFSID").value
            if WF.get_opt("debug"):
                logging.debug("sid from cookie: %s", str(self.sid))
        except KeyError:
            m = md5.md5(_sidhelper)
            m.update(str(time.time()))
            m.update(str(os.getpid()))
            m.update(str(os.getppid()))
            m.update(str(WF.get_opt("session.randomchars")))
            sid = m.hexdigest()
            
            # Add a bit more confusion for attackers            
            m.update(_sidhelper)
            _sidhelper = m.hexdigest()
            self.sid = sid
            logging.debug("created sid: %s", str(self.sid))

        ctx.resp.set_cookie("WFSID", self.sid,
                            time.time()+WF.get_opt("session.timeout"))

    def _load(self):
        """Loads the session.
        Is called automagically by the system"""
        dbh = WF.open_db()
        cr = dbh.simple(WF.get_opt("session.sql_get"), self.sid)
        if cr.rowcount == 1:
            data = cr.fetchone()
            data = data["data"]
            if not isinstance(data, str):
                data = data.tostring()
            data = cPickle.loads(data)
            self.update(data)

    def _save(self):
        """Saves the session.
        Is called automagically by the system"""
        dbh = WF.open_db()
        data = cPickle.dumps(self, protocol=-1)
        dbh.simple(WF.get_opt("session.sql_set"), data, self.sid)
        dbh.commit()
        

# Init sidhelper at startup
# This makes guessing of session ids a lot harder since attackers
# would have a hard time pinpointing the exact system clock when the
# module was initialized
_sidhelper = str(time.time())
            
